Attribute VB_Name = "Sorting"
Option Explicit

Private ParentIndex() As Long
Private NumChildren() As Long
Private NumParents As Long


' this is the public sort routine to be called for an LVItemTree
' it will sort the parent items only and leave the children untouched
' nevertheless all children stay behind their respective parent

Public Sub SortParentItems(LV As ListView)
  Dim i&
  
  IndicateParents LV 'make a list of all parent item indexes
  Debug.Print NumParents
  For i = 0 To NumParents - 1
   Debug.Print ParentIndex(i); ","; NumChildren(i); ","; LV.ListItems(ParentIndex(i))
  Next
  BubbleSortIndexes LV
  
  RearrangeItems LV
 
End Sub


Private Sub RearrangeItems(LV As ListView)
' this assumes that the index array has been sorted
  Dim i&, c&, n&, t&
  
  'first append all items in sorted order
  'c counts the duplicated items to be deleted afterwards
  For i = 0 To NumParents - 1
      DuplicateItem LV, ParentIndex(i)
      n = n + 1
      For c = 1 To NumChildren(i)
          DuplicateItem LV, ParentIndex(i) + c
          n = n + 1
      Next
  Next

  'now delete the unsorted bunch of items
  For i = 1 To n
      LV.ListItems.Remove 1
  Next
End Sub

Private Sub DuplicateItem(LV As ListView, iItem As Long)
' duplicates an indexed item to the end of the list
  Dim lit As ListItem 'to-item
  Dim lif As ListItem 'from-item
  Dim ind As Long
  Dim lvs As LVItemStates
  Dim s&
  
  Set lif = LV.ListItems(iItem)
  Set lit = LV.ListItems.Add(, , lif.Text, , lif.SmallIcon)
  
  'when duplicating an item we have to take care of the subitems/columns
  For s = 1 To LV.ColumnHeaders.Count - 1
      lit.SubItems(s) = lif.SubItems(s)
  Next
  
  'and we have to duplicate the extended item state or we lose indentation
  lvs = Listview_GetItemStateEx(LV.hWnd, iItem - 1, ind)
  Listview_SetItemStateEx LV.hWnd, lit.Index - 1, ind, lvs

End Sub

Private Sub IndicateParents(LV As ListView)
  'count the parent items in LV and make a list of parent indexes
  Dim p&, i&, n&
  Dim iInd
  For i = 1 To LV.ListItems.Count
      If IsParent(LV, i) Then p = p + 1
  Next
  NumParents = p
  
  ReDim ParentIndex(p - 1)
  ReDim NumChildren(p - 1)
  n = 0
  
  'collect all parent item indexes
  For i = 1 To LV.ListItems.Count
      If IsParent(LV, i) Then
         ParentIndex(n) = i 'store index of this parent
         If n > 0 Then
            'determine the number of children of the PREVIOUS parent
            NumChildren(n - 1) = ParentIndex(n) - ParentIndex(n - 1) - 1
         End If
         n = n + 1
      End If
  Next
  'get number of children of the last parent
  NumChildren(p - 1) = LV.ListItems.Count - ParentIndex(p - 1)

' now we have an array of indexes of all parent items and
' one array with all child items of each parent
  
End Sub

Private Function IsParent(LV As ListView, iItem As Long) As Boolean
' this returns true if the indentation of an item is 0, which indicates a parent
  Dim iIndent As Long
  Listview_GetItemStateEx LV.hWnd, iItem - 1, iIndent
  IsParent = iIndent < 1
End Function



Private Sub BubbleSortIndexes(LV As ListView)
' sorts parent indexes and their children count by matching the items
' this part can be done more sophisticated, I just use a very simple bubble sort
  
  Dim i&, j&, tmp&
  
  If NumParents < 2 Then Exit Sub 'nothing to sort then
  
  For i = 0 To NumParents - 2
      For j = 0 To NumParents - 2
          If LV.ListItems(ParentIndex(j)).Text > LV.ListItems(ParentIndex(j + 1)).Text Then
             'swap parent index
             tmp = ParentIndex(j)
             ParentIndex(j) = ParentIndex(j + 1)
             ParentIndex(j + 1) = tmp
             'swap children count
             tmp = NumChildren(j)
             NumChildren(j) = NumChildren(j + 1)
             NumChildren(j + 1) = tmp
           End If
      Next
  Next
End Sub


